import logging
import datetime
import os
from os.path import join as pjoin


log = logging.getLogger('doto')
log.setLevel(logging.DEBUG)

try:
  os.path.expanduser('~')
  expanduser = os.path.expanduser
except (AttributeError, ImportError):
  # This is probably running on App Engine.
  expanduser = (lambda x: x)

DOTO_DIR = pjoin(expanduser('~'),'.doto')
DOTO_LOG_DIR = pjoin(DOTO_DIR,'doto-logs')
now = datetime.datetime.utcnow().strftime('%Y-%m-%d')


if not os.path.exists(DOTO_LOG_DIR):
    os.makedirs(DOTO_LOG_DIR)

DEBUG_FILE = pjoin(DOTO_LOG_DIR, 'doto-%s.log' % (now))

#create file handler which logs even debug messages
fh = logging.FileHandler(DEBUG_FILE)
fh.setLevel(logging.DEBUG)

# create console handler with a higher log level
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)

# create formatter and add it to the handlers
file_formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
fh.setFormatter(file_formatter)

console_formatter = logging.Formatter('>>> %(message)s')

ch.setFormatter(console_formatter)

# add the handlers to the logger
log.addHandler(fh)
log.addHandler(ch)



from __future__ import print_function, division, absolute_import
import requests

from doto.logger import log

class DOError(Exception):
    pass

class d0mixin(object):

    def _request(self, event, status_check=None, **kwds):
            if 'client_id' not in kwds:
                kwds['client_id'] = self._client_id
            if 'api_key' not in kwds:
                kwds['api_key'] = self._api_key

            headers = {
                'User-Agent': 'doto/client'
            }

            for key, value in kwds.iteritems():
                log.debug("%s = %s" % (key, value))



            BASEURL = "https://api.digitalocean.com"
            response = requests.get(BASEURL+event,headers=headers,params=kwds)
            log.info('Getting '+event)
            log.debug(response.url)

            if response.status_code == 200:
                data = response.json()
                log.debug(data)

                if data['status'] == 'ERROR':
                    log.debug("Error with request: %s" % (data['message']))
                    error = "MSG: %s" % (data['message'])
                    raise DOError(error)

                if status_check:
                    return response.status_code

                return data
            else:
                #error
                data = response.json()
                error = "Status code: %d MSG: %s" % (response.status_code, data['message'])
                raise DOError(error)



from __future__ import print_function, division, absolute_import
import requests

from doto.logger import log
from doto.config import Config
from doto.droplet import Droplet
from doto.image import Image
from doto.domain import Domain
from doto.d0_mixin import d0mixin
from Crypto.PublicKey import RSA
import os
from os.path import join as pjoin

try:
  os.path.expanduser('~')
  expanduser = os.path.expanduser
except (AttributeError, ImportError):
  # This is probably running on App Engine.
  expanduser = (lambda x: x)

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

BASEURL = "https://api.digitalocean.com"

class connect_d0(d0mixin):

    def __init__(self, path=None,debug=True):

        #logging is off when log.disabled is set to True
        if not debug:
            log.disabled = True

        self.config = Config(path)
        self._client_id = self.config.get('Credentials','client_id')
        self._api_key = self.config.get('Credentials','api_key')

    def __str__(self):
        return "DigitialOcean Connection Object"

    def __repr__(self):
        return "D0:Connected"

    def _set_logging(self,debug=True):
        """
        Convenience function to set logging on/off
        """

        #logging is off when log.disabled is set to True
        if not debug:
            log.disabled = True

        else:
            log.disabled = False


    def _attach_auth(self,items):
        """
        convenience method to attach client_id and api_key to
        Image or Droplet object
        """

        for d in items:
            #convert dictionary to droplet objects
            d['_client_id'] = self._client_id
            d['_api_key'] = self._api_key

        return items

    def _pprint_table(self, data):
        """
        pprint table: from stackoverflow:
        http://stackoverflow.com/a/8356620
        """

        table = []
        for d in data:
            table.append([unicode(v) for v in d.values()])

        header = d.keys()
        table.insert(0,header)

        col_width = [max(len(x) for x in col) for col in zip(*table)]
        for line in table:
            print("| " + " | ".join("{:{}}".format(x, col_width[i])
                                    for i, x in enumerate(line)) + " |")




    def create_droplet(self,name=None,size_id=None,image_id=None,
                       region_id=None,ssh_key_ids=None,private_networking=None):
        """
        Creates a droplet

        :type name: string
        :param name: The NAME of the droplet (name will be used as a reference
        on D0's servers)

        :type size_id: int
        :param size_id: The ID corresponding to requested size of the image (see
        connect_d0.get_sizes)

        :type image_id: int
        :param image_id: The ID corresponding to the requested image (see
        connect_d0.images)

        :type region_id: int
        :param region_id: The ID corresponding to the requested region (see
        connect_d0.region_id)

        :type ssh_key_ids: int
        :param ssh_key_ids: An optional list of comma separated IDs corresponding
        to the requested ssh_keys to be added to the server
        (see connect_d0.get_ssh_keys)

        :type private_networking: int
        :param private_networking: An optional bool which enables a private network interface
        if the region supports private networking

        droplet = d0.create_droplet(name='Random',
                               size_id=66, #512MB
                               image_id=1341147, #Docker 0.7 Ubuntu 13.04 x64
                               region_id=1, #New York
                               ssh_key_ids=18669
                               )
        """

        #ssh_key_ids check/convert to string
        if isinstance(ssh_key_ids,(tuple,list)):
            ssh_key_ids = ', '.join(str(key) for key in ssh_key_ids)

        data = self._request("/droplets/new",name=name,
                          size_id=size_id,image_id=image_id,
                          region_id=region_id,ssh_key_ids=ssh_key_ids,
                          private_networking=private_networking)

        #don't like this but will do for now
        data['droplet']['_client_id'] = self._client_id
        data['droplet']['_api_key'] = self._api_key
        droplet = Droplet(**data['droplet'])

        droplet.update()
        droplet.event_update()

        return droplet

        # https://api.digitalocean.com/droplets/new?client_id=[your_client_id]&api_key=[your_api_key]&
        # name=[droplet_name]&size_id=[size_id]&image_id=[image_id]&region_id=[region_id]&ssh_key_ids=
        # [ssh_key_id1],[ssh_key_id2]

    def get_droplet_by_name(self, name):
        """
        Convenience method to make it easy to select a droplet by name
        """
        droplets = self.get_all_droplets()
        for d in droplets:
            if d.name == name:
              return d
        return None


    def get_all_droplets(self,filters=None, status_check=None, table=False, raw_data=False):
        """
        This method returns all active droplets that are currently running in your account.
        All available API information is presented for each droplet.

        https://api.digitalocean.com/droplets/?
        client_id=[your_client_id]&api_key=[your_api_key]

        :rtype: list
        :return: A list of :class:`doto.Droplet`
        """


        log.info("Get All Droplets")
        data = self._request("/droplets",status_check)

        if status_check:
            return data

        if raw_data:
            return data

        if table:
            self._pprint_table(data['droplets'])
            return

        droplets = self._attach_auth(data['droplets'])

        if filters:
            droplets = [Droplet(**drop) for drop in droplets]
            for k,v in filters.iteritems():
                droplets = filter(lambda x: v in getattr(x,k), droplets)

            return droplets


        #convert dictionary to droplet objects
        return [Droplet(**drop) for drop in droplets]

    def get_droplet(self, id=None):
        """
        This method returns full information for a specific droplet ID that is passed in the URL.

        :type id: int
        :param id: ID of the droplet

        https://api.digitalocean.com/droplets/[droplet_id]?
        client_id=[your_client_id]&api_key=[your_api_key]

        """

        data = self._request("/droplets/"+str(id))

        #don't like this but will do for now
        data['droplet']['_client_id'] = self._client_id
        data['droplet']['_api_key'] = self._api_key

        #convert dictionary to droplet objects
        return Droplet(**data['droplet'])


    def get_sizes(self,status_check=None, table=False):
        """
        This method returns all the available sizes that can be used to create a droplet.


        https://api.digitalocean.com/sizes/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        data = self._request("/sizes", status_check)

        if status_check:
            return data

        sizes = data['sizes']
        if table:
            self._pprint_table(sizes)

        return sizes


    def get_all_regions(self,status_check=None, table=False):
        """
        This method will return all the available regions within the DigitalOcean cloud.

        https://api.digitalocean.com/sizes/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        data = self._request("/regions", status_check)

        if status_check:
            return data

        regions = data['regions']


        if table:
            self._pprint_table(regions)

        return  regions

    def get_domains(self, status_check=None, table=False):
        """
        This method returns all of your current domains.

        https://api.digitalocean.com/domains/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """


        data = self._request("/domains", status_check)
        if status_check:
            return data

        domains = data['domains']

        if table:
            self._pprint_table(domains)

        return domains

    def get_all_ssh_keys(self, status_check=None, table=False):
        """
        This method lists all the available public SSH keys in
        your account that can be added to a droplet.

        https://api.digitalocean.com/ssh_keys/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """


        data = self._request("/ssh_keys", status_check)

        if status_check:
            return data

        sshkeys = data['ssh_keys']

        if table:
            self._pprint_table(sshkeys)

        return sshkeys

    def create_key_pair(self, ssh_key_name=None, dry_run=False):
        """
        Method to create a key pair and store the public key on Digital Ocean's servers

        :type ssh_key_name: string
        :param ssh_key_name: The name of the new keypair

        :type dry_run: bool
        :param dry_run: Set to True if the operation should not actually run.

        :rtype: dict
        :return: Dictionary of SSH key info and local path

        https://api.digitalocean.com/ssh_keys/new/?name=[ssh_key_name]&ssh_pub_key=[ssh_public_key]&
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        ssh_path = pjoin(expanduser('~'), '.ssh')

        if not os.path.isdir(ssh_path): os.makedirs(ssh_path)

        #store key file in ~/.ssh
        keyfile = pjoin(ssh_path, ssh_key_name)

        key = RSA.generate(2048,os.urandom)

        #public key
        with open(keyfile+'_rsa.pub','w') as f:
            f.write(key.exportKey('OpenSSH'))
        public_key = key.exportKey('OpenSSH')
        os.chmod(keyfile+'_rsa.pub', 0o0600)

        #private key
        with open(keyfile+'_rsa','w') as f:
            f.write(key.exportKey())

        os.chmod(keyfile+'_rsa', 0o0600)

        data = self._request("/ssh_keys/new/", name=ssh_key_name,
                             ssh_pub_key=public_key)

        #include path to newly created file
        data['ssh_key']['path'] = keyfile+'_rsa'

        log.info(data['ssh_key'])
        return data['ssh_key']

    def delete_key_pair(self, ssh_key_id=None):
        """
        Delete the SSH key from your account.

        :type ssh_key_id: int
        :param ssh_key_id: The ID of the public key

        https://api.digitalocean.com/ssh_keys/[ssh_key_id]/destroy/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/ssh_keys/%d/destroy" % (ssh_key_id)

        data = self._request(url)

        log.info(data)

    def get_ssh_key(self, ssh_key_id=None):
        """
        Delete the SSH key from your account.

        :type ssh_key_id: int
        :param ssh_key_id: The ID of the public key

        https://api.digitalocean.com/ssh_keys/[ssh_key_id]/destroy/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/ssh_keys/%d" % (ssh_key_id)

        data = self._request(url)

        log.info(data)

    def get_all_images(self, filters=None, status_check=False, table=False, raw_data=False):
        """
        Convenience method to get Digital Ocean's list of public images
        and users current private images
        using EC2 style filtering

        https://api.digitalocean.com/sizes/?client_id=[your_client_id]&api_key=[your_api_key]

        :type filters: dict
        :param filters: Optional filters that can be used to limit the
            results returned.  Filters are provided in the form of a
            dictionary consisting of filter names as the key and
            filter values as the value.  The set of allowable filter
            names/values is dependent on the request being performed.
            Check the DigitalOcean API guide for details.

        :rtype: list
        :return: A list of :class:`doto.Image` objects


        https://api.digitalocean.com/images/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        data = self._request("/images", status_check)

        if raw_data:
            return data

        if status_check:
            return data

        if table:
            self._pprint_table(data['images'])
            return

        images = self._attach_auth(data['images'])


        if filters:
            images = [Image(**img) for img in images]
            for k,v in filters.iteritems():
                images = filter(lambda x: v in getattr(x,k), images)

            return images

        #convert dictionary to Image objects
        return [Image(**img) for img in images]


    def get_image(self, image_id=None):
        """
        This method displays the attributes of an image.

        :type image_id: int
        :param image_id: The ID of the image

        :rtype: :class:`doto.Image`
        :return: The newly created :class:`doto.Image`.

        https://api.digitalocean.com/images/[image_id]/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/images/%d" % (image_id)

        data = self._request(url)


        log.info(data)

        #don't like this but will do for now
        data['image']['_client_id'] = self._client_id
        data['image']['_api_key'] = self._api_key


        return Image(**data['image'])

    def get_all_domains(self,filters=None, status_check=None, table=False, raw_data=False):
        """
        This method returns all active droplets that are currently running in your account.
        All available API information is presented for each droplet.

        https://api.digitalocean.com/droplets/?
        client_id=[your_client_id]&api_key=[your_api_key]

        :rtype: list
        :return: A list of :class:`doto.Droplet`
        """

        data = self._request("/domains",status_check)

        if status_check:
            return data

        if raw_data:
            return data

        if table:
            self._pprint_table(data['domains'])
            return

        domains = self._attach_auth(data['domains'])

        if filters:
            droplets = [Droplet(**drop) for drop in domains]
            for k,v in filters.iteritems():
                droplets = filter(lambda x: v in getattr(x,k), droplets)

            return droplets


        #convert dictionary to droplet objects
        return [Domain(**dom) for dom in domains]

    def create_domain(self,name=None,ip_addr=None):
        """
        This method creates a new domain name with an A record for the specified [ip_address].

        :type name: string
        :param size: The NAME of the domain


        :type ip_address: string
        :param size: ip address for the domain's initial a record.

        https://api.digitalocean.com/domains/new?
        client_id=[your_client_id]&api_key=[your_api_key]&name=[domain]&ip_address=[ip_address]
        """


        log.info("Creating new domain")

        data = self._request("/domains/new",name=name,
                          ip_address=ip_addr)

        log.debug(data)

        #don't like this but will do for now
        data['domain']['_client_id'] = self._client_id
        data['domain']['_api_key'] = self._api_key
        domain = Domain(**data['domain'])

        return domain

    def get_domain(self, domain_id=None):
        """
        This method displays the attributes of an image.

        :type image_id: int
        :param image_id: The ID of the image

        :rtype: :class:`doto.Image`
        :return: The newly created :class:`doto.Image`.

        https://api.digitalocean.com/domains/[domain_id]?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/domains/%d" % (domain_id)

        data = self._request(url)


        log.debug(data)

        #don't like this but will do for now
        data['domain']['_client_id'] = self._client_id
        data['domain']['_api_key'] = self._api_key


        return Domain(**data['domain'])




import os
from os.path import join as pjoin
import six
from six.moves import configparser


# If running in Google App Engine there is no "user" and
# os.path.expanduser() will fail. Attempt to detect this case and use a
# no-op expanduser function in this case.
try:
  os.path.expanduser('~')
  expanduser = os.path.expanduser
except (AttributeError, ImportError):
  # This is probably running on App Engine.
  expanduser = (lambda x: x)

# By default we use two locations for the doto configurations,
# /etc/Doto.cfg and ~/.doto (which works on Windows and Unix).
DotoConfigPath = '/etc/dotorc.cfg'
DotoConfigLocations = [DotoConfigPath]
UserConfigPath = pjoin(expanduser('~'),'.doto', '.dotorc')
DotoConfigLocations.append(UserConfigPath)

# If there's a DOTO_CONFIG variable set, we load ONLY
# that variable
if 'DOTO_CONFIG' in os.environ:
    DotoConfigLocations = [expanduser(os.environ['DOTO_CONFIG'])]

# If there's a DOTO_PATH variable set, we use anything there
# as the current configuration locations, split with colons
elif 'DOTO_PATH' in os.environ:
    DotoConfigLocations = []
    for path in os.environ['DOTO_PATH'].split(":"):
        DotoConfigLocations.append(expanduser(path))

class Config(configparser.SafeConfigParser):

    def __init__(self, path=None,):
        if six.PY3:
            super(Config,self).__init__(allow_no_value=True)
        else:
            # We don't use ``super`` here, because ``ConfigParser`` still uses
            # old-style classes.
            configparser.SafeConfigParser.__init__(self, allow_no_value=True)

        if path:
            self.read(path)
            self.file_path = path
        else:
            f = self.read(DotoConfigLocations)
            self.file_path = f[0]

    def get(self, section, name, default=None):
        try:
            val = configparser.SafeConfigParser.get(self, section, name)
        except:
            val = default
        return val




# This file was generated by 'versioneer.py' (0.7+) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = 'v0.2'
version_full = '2895ea7e0bcf6b16c5e5bc98a10fd46d8e05e172'
def get_versions(default={}, verbose=False):
    return {'version': version_version, 'full': version_full}




from __future__ import print_function, division, absolute_import

from doto.logger import log
from doto.d0_mixin import d0mixin

import requests


class Droplet(d0mixin):

    def __str__(self):
        return ("Droplet:%s") % (self.id)

    def __repr__(self):
        return ("Droplet:%s") % (self.id)

    def __init__(self, **kwds):
        self.__dict__.update(kwds)

    def update(self):
        """
        Method to update Droplet (primarily used to update ip information and state changes)

        https://api.digitalocean.com/events/[event_id]/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        data = self._request("/droplets/"+str(self.id))

        self.__dict__.update(**data['droplet'])

    def event_update(self):
        """
        Method to update Droplet

        https://api.digitalocean.com/events/[event_id]/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/events/%s" % (str(self.event_id))
        data = self._request(url)
        log.debug("Updating Event")
        log.debug(data)

        data['event']['event_id'] = data['event']['id']
        data['event']['id'] = data['event']['droplet_id']

        self.__dict__.update(**data['event'])

    def percentage_update(self):
        """
        Convenience method to return the percentage of event completion

        """

        #needed to grab ip_address
        self.update()

        self.event_update()
        return self.percentage

    def rename(self,name=None):
        """
        This method renames the droplet to the specified name.

        :type name: str
        :param name: Name of the new droplet

        https://api.digitalocean.com/droplets/[droplet_id]/rename/?
        client_id=[your_client_id]&api_key=[your_api_key]&name=[name]
        """


        url = "/droplets/%s/rename" % (str(self.id))

        data = self._request(url,name=name)

        self.event_id = data['event_id']

        log.info("Renaming: %d To: %s Event: %d" % (self.id, name, self.event_id))
        self.update()

    def rebuild(self, image_id=None,use_current=False):

        """
        This method allows you to reinstall a droplet with a default image.
        This is useful if you want to start again but retain the same IP address for your droplet.

        :type image_id: int
        :param image_id: ID of the image you would like to use to rebuild your droplet with

        :type use_current: BOOL
        :param use_current: Use the current image_id of the droplet during rebuild process

        https://api.digitalocean.com/droplets/[droplet_id]/rebuild/?image_id=[image_id]&
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        if use_current:
            image_id = self.image_id

        url = "/droplets/%s/rebuild" % (str(self.id))

        data = self._request(url,image_id=image_id)

        self.event_id = data['event_id']

        self.update()
        log.info("Rebuild: %d With: %d Event: %d" % (self.id, image_id, self.event_id))

    def restore(self, image_id=None):

        """
        This method allows you to restore a droplet with a previous image or snapshot.
        This will be a mirror copy of the image or snapshot to your droplet.
        Be sure you have backed up any necessary information prior to restore.


        :type image_id: int
        :param image_id: ID of the image you would like to use to rebuild your droplet with

        https://api.digitalocean.com/droplets/[droplet_id]/restore/?image_id=[image_id]&
        client_id=[your_client_id]&api_key=[your_api_key]
        """



        url = "/droplets/%s/restore" % (str(self.id))

        data = self._request(url,image_id=image_id)

        self.event_id = data['event_id']

        log.info("Restoring: %d With: %d Event: %d" % (self.id, image_id, self.event_id))
        self.update()


    def set_backups(self,flag=True):
        """
        This method enables/disables automatic backups
        which run in the background daily to backup your droplet's data.


        :type flag: bool
        :param scrub_data: A bool which enables/disables backups

        https://api.digitalocean.com/droplets/[droplet_id]/enable_backups/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """


        backup_setting = "enable_backups" if flag else "disable_backups"

        url = "/droplets/%s/%s" % (str(self.id), backup_setting)

        data = self._request(url)

        self.event_id = data['event_id']

        log.info("Destroying: %d, Event: %d" % (self.id, self.event_id))


    def destroy(self, scrub_data=1):
        """
        This method destroys one of your droplets - this is irreversible.

        :type scrub_data: bool
        :param scrub_data: An optional bool which will strictly write 0s to your prior
        partition to ensure that all data is completely erased. True by default

        https://api.digitalocean.com/droplets/[droplet_id]/destroy/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/droplets/%s/destroy" % (str(self.id))

        data = self._request(url,scrub_data=scrub_data)

        self.event_id = data['event_id']

        log.info("Destroying: %d, Event: %d" % (self.id, self.event_id))

    def reboot(self):
        """
        This method allows you to reboot a droplet.
        This is the preferred method to use if a server is not responding.

        https://api.digitalocean.com/droplets/[droplet_id]/reboot/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/droplets/%s/reboot" % (str(self.id))

        data = self._request(url)

        self.event_id = data['event_id']

        log.info("Rebooting: %d, Event: %d" % (self.id, self.event_id))

    def shutdown(self):
        """
        This method allows you to shutdown a droplet.
        The droplet will remain in your account.

        https://api.digitalocean.com/droplets/[droplet_id]/shutdown/
        ?client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/droplets/%s/shutdown" % (str(self.id))

        data = self._request(url)

        self.event_id = data['event_id']

        log.info("Shutting Down: %d, Event: %d" % (self.id, self.event_id))
        log.info("Droplet remains active in your account")



    def power_cycle(self):
        """
        This method allows you to power cycle a droplet.
        This will turn off the droplet and then turn it back on.

        https://api.digitalocean.com/droplets/[droplet_id]/power_cycle/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/droplets/%s/power_cycle" % (str(self.id))

        data = self._request(url)

        self.event_id = data['event_id']

        log.info("Power Cycle: %d, Event: %d" % (self.id, self.event_id))

    def power_off(self):
        """
        This method allows you to power off a droplet.
        This will turn off the droplet and then turn it back on.

        https://api.digitalocean.com/droplets/[droplet_id]/power_off/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/droplets/%s/power_off" % (str(self.id))

        data = self._request(url)

        self.event_id = data['event_id']

        log.info("Powering Off: %d, Event: %d" % (self.id, self.event_id))



    def power_on(self):
        """
        This method allows you to power on a previously powered off droplet.

        https://api.digitalocean.com/droplets/[droplet_id]/power_on/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/droplets/%s/power_on" % (str(self.id))

        data = self._request(url)

        self.event_id = data['event_id']

        log.info("Powering On: %d, Event: %d" % (self.id, self.event_id))


    def password_reset(self):
        """
        This method will reset the root password for a droplet.
        Please be aware that this will reboot the droplet to allow resetting the password.

        https://api.digitalocean.com/droplets/[droplet_id]/password_reset/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/droplets/%s/password_reset" % (str(self.id))

        data = self._request(url)

        self.event_id = data['event_id']

        log.info("Resetting Password: %d, Event: %d" % (self.id, self.event_id))
        log.info("Rebooting Droplet")


    def resize(self,size=None):
        """
        This method allows you to resize a specific droplet to a different size.
        This will affect the number of processors and memory allocated to the droplet.

        REQUIRES SNAPSHOT OF DROPLET

        :type size: int
        :param size: The new SIZE id of the droplet


        https://api.digitalocean.com/droplets/[droplet_id]/resize/?size_id=[size_id]&
        client_id=[your_client_id]&api_key=[your_api_key]
        """


        url = "/droplets/%s/resize" % (str(self.id))

        data = self._request(url,size=size)

        self.event_id = data['event_id']

        log.info("Resizing Droplet: %d, Event: %d" % (self.id, self.event_id))
        log.info("Rebooting Droplet")

    def create_snapshot(self,name=None):
        """
        This method allows you to take a snapshot of the droplet once it has been powered off,
        which can later be restored or used to create a new droplet from the same image.
        Please be aware this may cause a reboot.

        :type name: string
        :param size: The NAME of the snapshot

        https://api.digitalocean.com/droplets/[droplet_id]/snapshot/?name=[snapshot_name]&
        client_id=[your_client_id]&api_key=[your_api_key]
        """


        #should thread this in the future
        log.info("Powering off %d" % (self.id))
        self.power_off()
        while self.status != 'off':
            self.update()

        url = "/droplets/%s/snapshot" % (str(self.id))

        data = self._request(url,name=name)

        self.event_id = data['event_id']

        log.info("Taking Snapshot: %d, Event: %d" % (self.id, self.event_id))



from __future__ import print_function, division, absolute_import

from doto.logger import log
from doto.d0_mixin import d0mixin

import requests


class Domain(d0mixin, object):

    def __str__(self):
        return ("Domain:%s") % (self.id)

    def __repr__(self):
        return ("Domain:%s") % (self.id)

    def __init__(self, **kwds):
        self.__dict__.update(kwds)

    def _pprint_table(self, data):
        """
        pprint table: from stackoverflow:
        http://stackoverflow.com/a/8356620
        """

        table = []
        for d in data:
            table.append([unicode(v) for v in d.values()])

        header = d.keys()
        table.insert(0,header)

        col_width = [max(len(x) for x in col) for col in zip(*table)]
        for line in table:
            print("| " + " | ".join("{:{}}".format(x, col_width[i])
                                    for i, x in enumerate(line)) + " |")


    def destroy(self):
        """
        This method allows you to destroy a domain.

        https://api.digitalocean.com/domains/[domain_id]/destroy/?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "/domains/%s/destroy" % (str(self.id))

        data = self._request(url)

        log.info(data)


    def get_all_records(self,table=False):
        """
        This method returns all of your current domain records.

        https://api.digitalocean.com/domains/[domain_id]/records?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        log.info("Getting Records")
        url = "/domains/%s/records" % (str(self.id))

        data = self._request(url)

        log.debug(data)

        if table:
            self._pprint_table(data['records'])

        return data['records']

    def get_record(self,record_id=None):
        """
        This method returns the specified domain record.

        :type record_id: int
        :param record_id: ID of record you are trying to retrieve.

        https://api.digitalocean.com/domains/[domain_id]/records/[record_id]?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        log.info("Getting Record: %d" (record_id))
        url = "/domains/%s/records/%d" % (str(self.id), record_id)

        data = self._request(url)

        log.debug(data)

        return data['record']

    def edit_record(self,record_id=None, record_type=None, data=None,
                         name=None,priority=None,port=None,weight=None,
                         ):
        """
        This method edits an existing domain record.

        :type record_id: int
        :param record_id: ID of record you are trying to retrieve.

        :type record_type: String
        :param record_type: the type of record you would like to create. 'A', 'CNAME', 'NS', 'TXT', 'MX' or 'SRV'.

        :type data: String
        :param data: This is the value of the record.

        :type name: String
        :param name: Required for 'A', 'CNAME', 'TXT' and 'SRV' records otherwise optional

        :type priority: int
        :param priority: required for 'SRV' and 'MX' records otherwise optional

        :type port: int
        :param port: required for 'SRV' records otherwise optional

        :type weight: int
        :param weight: required for 'SRV' records. otherwise optional

        https://api.digitalocean.com/domains/[domain_id]/records/[record_id]/edit?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        log.info("Editing Record: %d" (record_id))
        url = "/domains/%s/records/%d/edit" % (str(self.id), record_id)

        data = self._request(url,record_type=record_type,
                             data=data,name=name,priority=priority,
                             port=port,weight=weight)

        log.debug(data)

        return data['record']

    def record_destroy(self, record_id=None):
        """
        This method deletes the specified domain record.

        https://api.digitalocean.com/domains/[domain_id]/records/[record_id]/destroy?
        client_id=[your_client_id]&api_key=[your_api_key]
        """

        url = "domains/%s/records/%d/destroy" % (str(self.id),record_id)

        data = self._request(url)

        log.info(data)






import sys

from distutils.core import setup

import versioneer


versioneer.versionfile_source = 'doto/_version.py'
versioneer.versionfile_build = 'doto/_version.py'
versioneer.tag_prefix = ''
versioneer.parentdir_prefix = 'doto-'

setup(
    name = 'doto',
    version = versioneer.get_version(),
    cmdclass = versioneer.get_cmdclass(),
    description='Python Interface to Digital Ocean',
    author = 'Benjamin Zaitlen',
    packages = ['doto'],
    install_requires=['requests>=2.0.1',
                      'pycrypto>=2.6.1'],
    package_data={'doto':['dotorc']},
)